"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const aws_sdk_1 = require("aws-sdk");
const errors_1 = require("./errors");
const getLogger_1 = __importDefault(require("./getLogger"));
const d = getLogger_1.default().extend("DynamoDBConnectionManager");
class ConnectionNotFoundError extends errors_1.ExtendableError {
}
exports.ConnectionNotFoundError = ConnectionNotFoundError;
class DynamoDBConnectionManager {
    constructor({ connectionsTable = "Connections", subscriptionsTable = "Subscriptions" } = {}) {
        this.hydrateConnection = async (connectionId) => {
            // if connection is not found, throw so we can terminate connection
            d("hydrateConnection called");
            d(`getting connection for id ${connectionId}...`);
            const result = await this.db
                .get({
                TableName: this.connectionsTable,
                Key: {
                    id: connectionId
                }
            })
                .promise();
            d(`getting connection for id ${connectionId}...done`);
            if (result.Item == null) {
                d(`connection with id ${connectionId} not found`);
                throw new ConnectionNotFoundError(`Connection ${connectionId} not found`);
            }
            d("returning connection");
            return result.Item;
        };
        this.registerConnection = async ({ connectionId, endpoint }) => {
            d("registerConnection");
            const connection = { id: connectionId, data: { endpoint } };
            d(`inserting connection to table ${this.connectionsTable}...`);
            await this.db
                .put({
                TableName: this.connectionsTable,
                Item: {
                    createdAt: new Date(),
                    id: connection.id,
                    data: connection.data
                }
            })
                .promise();
            d(`inserting connection to table ${this.connectionsTable}...`);
            return connection;
        };
        this.sendToConnection = async (connection, payload) => {
            d("sendToConnection");
            const { data: { endpoint }, id } = connection;
            const managementApi = new aws_sdk_1.ApiGatewayManagementApi({
                endpoint,
                apiVersion: "2018-11-29"
            });
            try {
                d("postToConnection");
                await managementApi.postToConnection({ ConnectionId: id, Data: payload }).promise();
            }
            catch (e) {
                d(`postToConnecticn failed `, e);
                // this is stale connection
                // remove it from DB
                if (e && e.statusCode === 410) {
                    d(`received 410 (gone)`, e);
                    await this.unregisterConnection(connection);
                }
                else {
                    throw e;
                }
            }
        };
        this.unregisterConnection = async (connection) => {
            d("unregisterConnection");
            d(`deleting connection ${connection.id} from ${this.connectionsTable}...`);
            await this.db
                .delete({
                Key: {
                    id: connection.id
                },
                TableName: this.connectionsTable
            })
                .promise();
            d(`deleting connection ${connection.id} from ${this.connectionsTable}...done`);
            d(`get subscriptions for connection ${connection.id} from ${this.subscriptionsTable}...`);
            const subscriptions = await this.db
                .query({
                TableName: this.subscriptionsTable,
                IndexName: "connectionId_gsi",
                KeyConditionExpression: "connectionId = :connectionId",
                ExpressionAttributeValues: { ":connectionId": connection.id },
                ProjectionExpression: "event,subscriptionId"
            })
                .promise();
            d(`get subscriptions for connection ${connection.id} from ${this.subscriptionsTable}...done`);
            d(`deleting subscriptions ...`);
            const deleteRequests = subscriptions.Items.map(item => ({
                DeleteRequest: {
                    Key: {
                        event: item.event,
                        subscriptionId: item.subscriptionId
                    }
                }
            }));
            await this.db
                .batchWrite({
                RequestItems: {
                    [this.subscriptionsTable]: deleteRequests
                }
            })
                .promise();
            d(`deleting subscriptions... done`);
        };
        d("constructor called", connectionsTable, subscriptionsTable);
        this.connectionsTable = connectionsTable;
        this.subscriptionsTable = subscriptionsTable;
        d("instantiating new DynamocDB DocumentClient...");
        this.db = new aws_sdk_1.DynamoDB.DocumentClient();
        d("instantiating new DynamocDB DocumentClient done");
    }
}
exports.DynamoDBConnectionManager = DynamoDBConnectionManager;
exports.default = DynamoDBConnectionManager;
